@@ -30,7 +30,7 @@ Ext.m3.ObjectGrid = Ext.extend(Ext.m3.GridPanel, {
 		Ext.m3.ObjectGrid.superclass.initComponent.call(this);
 		var store = this.getStore();
 		store.baseParams = Ext.applyIf(store.baseParams || {}, this.actionContextJson || {});
-
+        this.on('beforestatesave', this.onBeforeStateSave);
 
 		this.addEvents(
 			/**
@@ -406,8 +406,18 @@ Ext.m3.ObjectGrid = Ext.extend(Ext.m3.GridPanel, {
         }
 		return baseConf;
     }
+    /**
+     * Запуск перерисовки грида, чтобы поля
+     * фильтрации принимали корректные размеры
+     * при динамическом добавлении
+     */
+    ,onBeforeStateSave: function(){
+        this.fireEvent('resize');
+    }
 });
 
+Ext.reg('objectgrid', Ext.m3.ObjectGrid);
+
 Ext.m3.EditorObjectGrid = Ext.extend(Ext.m3.EditorGridPanel, {
 	constructor: function(baseConfig, params){
 
@@ -680,3 +690,5 @@ Ext.m3.EditorObjectGrid = Ext.extend(Ext.m3.EditorGridPanel, {
 		return baseConf;
     }
 });
+
+Ext.reg('editorobjectgrid', Ext.m3.EditorObjectGrid);
